# coding: utf-8

"""
    Cyberwave API

    Cyberwave platform REST API. Use 'Bearer YOUR_TOKEN' for authentication.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import StrictStr
from typing import List, Optional
from cyberwave.rest.models.cloud_node_workload_assign_schema import CloudNodeWorkloadAssignSchema
from cyberwave.rest.models.cloud_node_workload_attachment_response_schema import CloudNodeWorkloadAttachmentResponseSchema
from cyberwave.rest.models.cloud_node_workload_attachment_schema import CloudNodeWorkloadAttachmentSchema
from cyberwave.rest.models.cloud_node_workload_create_schema import CloudNodeWorkloadCreateSchema
from cyberwave.rest.models.cloud_node_workload_result_schema import CloudNodeWorkloadResultSchema
from cyberwave.rest.models.cloud_node_workload_schema import CloudNodeWorkloadSchema
from cyberwave.rest.models.cloud_node_workload_update_schema import CloudNodeWorkloadUpdateSchema
from cyberwave.rest.models.deferred_task_execution_response_schema import DeferredTaskExecutionResponseSchema

from cyberwave.rest.api_client import ApiClient, RequestSerialized
from cyberwave.rest.api_response import ApiResponse
from cyberwave.rest.rest import RESTResponseType


class CloudNodeWorkloadsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def src_app_api_cloud_node_workloads_assign_workload_to_instance(
        self,
        uuid: StrictStr,
        cloud_node_workload_assign_schema: CloudNodeWorkloadAssignSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadSchema:
        """Assign Workload To Instance

        Assign a workload to a Cloud Node instance.  This also updates the workload status to ASSIGNED.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_assign_schema: (required)
        :type cloud_node_workload_assign_schema: CloudNodeWorkloadAssignSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_assign_workload_to_instance_serialize(
            uuid=uuid,
            cloud_node_workload_assign_schema=cloud_node_workload_assign_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_assign_workload_to_instance_with_http_info(
        self,
        uuid: StrictStr,
        cloud_node_workload_assign_schema: CloudNodeWorkloadAssignSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadSchema]:
        """Assign Workload To Instance

        Assign a workload to a Cloud Node instance.  This also updates the workload status to ASSIGNED.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_assign_schema: (required)
        :type cloud_node_workload_assign_schema: CloudNodeWorkloadAssignSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_assign_workload_to_instance_serialize(
            uuid=uuid,
            cloud_node_workload_assign_schema=cloud_node_workload_assign_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_assign_workload_to_instance_without_preload_content(
        self,
        uuid: StrictStr,
        cloud_node_workload_assign_schema: CloudNodeWorkloadAssignSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Assign Workload To Instance

        Assign a workload to a Cloud Node instance.  This also updates the workload status to ASSIGNED.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_assign_schema: (required)
        :type cloud_node_workload_assign_schema: CloudNodeWorkloadAssignSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_assign_workload_to_instance_serialize(
            uuid=uuid,
            cloud_node_workload_assign_schema=cloud_node_workload_assign_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_assign_workload_to_instance_serialize(
        self,
        uuid,
        cloud_node_workload_assign_schema,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if cloud_node_workload_assign_schema is not None:
            _body_params = cloud_node_workload_assign_schema


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads/{uuid}/assign',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_create_workload(
        self,
        cloud_node_workload_create_schema: CloudNodeWorkloadCreateSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadSchema:
        """Create Workload

        Create a new Cloud Node workload.  The workload will be created in PENDING status and can later be assigned to an available Cloud Node instance.

        :param cloud_node_workload_create_schema: (required)
        :type cloud_node_workload_create_schema: CloudNodeWorkloadCreateSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_create_workload_serialize(
            cloud_node_workload_create_schema=cloud_node_workload_create_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_create_workload_with_http_info(
        self,
        cloud_node_workload_create_schema: CloudNodeWorkloadCreateSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadSchema]:
        """Create Workload

        Create a new Cloud Node workload.  The workload will be created in PENDING status and can later be assigned to an available Cloud Node instance.

        :param cloud_node_workload_create_schema: (required)
        :type cloud_node_workload_create_schema: CloudNodeWorkloadCreateSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_create_workload_serialize(
            cloud_node_workload_create_schema=cloud_node_workload_create_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_create_workload_without_preload_content(
        self,
        cloud_node_workload_create_schema: CloudNodeWorkloadCreateSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create Workload

        Create a new Cloud Node workload.  The workload will be created in PENDING status and can later be assigned to an available Cloud Node instance.

        :param cloud_node_workload_create_schema: (required)
        :type cloud_node_workload_create_schema: CloudNodeWorkloadCreateSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_create_workload_serialize(
            cloud_node_workload_create_schema=cloud_node_workload_create_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_create_workload_serialize(
        self,
        cloud_node_workload_create_schema,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if cloud_node_workload_create_schema is not None:
            _body_params = cloud_node_workload_create_schema


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_delete_workload(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete Workload

        Delete a Cloud Node workload.  Only pending or completed workloads can be deleted.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_delete_workload_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_delete_workload_with_http_info(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete Workload

        Delete a Cloud Node workload.  Only pending or completed workloads can be deleted.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_delete_workload_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_delete_workload_without_preload_content(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete Workload

        Delete a Cloud Node workload.  Only pending or completed workloads can be deleted.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_delete_workload_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_delete_workload_serialize(
        self,
        uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v1/cloud-node-workloads/{uuid}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload(
        self,
        uuid: StrictStr,
        cloud_node_workload_attachment_schema: CloudNodeWorkloadAttachmentSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadAttachmentResponseSchema:
        """Get Signed Url For Attachment Upload

        Generate a signed URL for uploading an attachment to a Cloud Node workload.  Returns a signed URL that can be used to upload a file directly to Google Cloud Storage.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_attachment_schema: (required)
        :type cloud_node_workload_attachment_schema: CloudNodeWorkloadAttachmentSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload_serialize(
            uuid=uuid,
            cloud_node_workload_attachment_schema=cloud_node_workload_attachment_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadAttachmentResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload_with_http_info(
        self,
        uuid: StrictStr,
        cloud_node_workload_attachment_schema: CloudNodeWorkloadAttachmentSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadAttachmentResponseSchema]:
        """Get Signed Url For Attachment Upload

        Generate a signed URL for uploading an attachment to a Cloud Node workload.  Returns a signed URL that can be used to upload a file directly to Google Cloud Storage.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_attachment_schema: (required)
        :type cloud_node_workload_attachment_schema: CloudNodeWorkloadAttachmentSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload_serialize(
            uuid=uuid,
            cloud_node_workload_attachment_schema=cloud_node_workload_attachment_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadAttachmentResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload_without_preload_content(
        self,
        uuid: StrictStr,
        cloud_node_workload_attachment_schema: CloudNodeWorkloadAttachmentSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Signed Url For Attachment Upload

        Generate a signed URL for uploading an attachment to a Cloud Node workload.  Returns a signed URL that can be used to upload a file directly to Google Cloud Storage.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_attachment_schema: (required)
        :type cloud_node_workload_attachment_schema: CloudNodeWorkloadAttachmentSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload_serialize(
            uuid=uuid,
            cloud_node_workload_attachment_schema=cloud_node_workload_attachment_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadAttachmentResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_get_signed_url_for_attachment_upload_serialize(
        self,
        uuid,
        cloud_node_workload_attachment_schema,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if cloud_node_workload_attachment_schema is not None:
            _body_params = cloud_node_workload_attachment_schema


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads/{uuid}/attachments',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_get_workload(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadSchema:
        """Get Workload

        Get a specific Cloud Node workload by UUID.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_workload_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_get_workload_with_http_info(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadSchema]:
        """Get Workload

        Get a specific Cloud Node workload by UUID.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_workload_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_get_workload_without_preload_content(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Workload

        Get a specific Cloud Node workload by UUID.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_workload_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_get_workload_serialize(
        self,
        uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/cloud-node-workloads/{uuid}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_get_workload_execution(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> DeferredTaskExecutionResponseSchema:
        """Get Workload Execution

        Get the execution information of a Cloud Node workload.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_workload_execution_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DeferredTaskExecutionResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_get_workload_execution_with_http_info(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[DeferredTaskExecutionResponseSchema]:
        """Get Workload Execution

        Get the execution information of a Cloud Node workload.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_workload_execution_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DeferredTaskExecutionResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_get_workload_execution_without_preload_content(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Workload Execution

        Get the execution information of a Cloud Node workload.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_get_workload_execution_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DeferredTaskExecutionResponseSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_get_workload_execution_serialize(
        self,
        uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads/{uuid}/execution',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_list_pending_workloads(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[CloudNodeWorkloadSchema]:
        """List Pending Workloads

        List all pending Cloud Node workloads visible to the authenticated user.  Useful for finding workloads that need to be assigned to instances.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_pending_workloads_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[CloudNodeWorkloadSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_list_pending_workloads_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[CloudNodeWorkloadSchema]]:
        """List Pending Workloads

        List all pending Cloud Node workloads visible to the authenticated user.  Useful for finding workloads that need to be assigned to instances.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_pending_workloads_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[CloudNodeWorkloadSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_list_pending_workloads_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Pending Workloads

        List all pending Cloud Node workloads visible to the authenticated user.  Useful for finding workloads that need to be assigned to instances.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_pending_workloads_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[CloudNodeWorkloadSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_list_pending_workloads_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/cloud-node-workloads/pending',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_list_task_executions(
        self,
        workspace_uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[DeferredTaskExecutionResponseSchema]:
        """List Task Executions

        List all executions for a workspace.

        :param workspace_uuid: (required)
        :type workspace_uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_task_executions_serialize(
            workspace_uuid=workspace_uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[DeferredTaskExecutionResponseSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_list_task_executions_with_http_info(
        self,
        workspace_uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[DeferredTaskExecutionResponseSchema]]:
        """List Task Executions

        List all executions for a workspace.

        :param workspace_uuid: (required)
        :type workspace_uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_task_executions_serialize(
            workspace_uuid=workspace_uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[DeferredTaskExecutionResponseSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_list_task_executions_without_preload_content(
        self,
        workspace_uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Task Executions

        List all executions for a workspace.

        :param workspace_uuid: (required)
        :type workspace_uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_task_executions_serialize(
            workspace_uuid=workspace_uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[DeferredTaskExecutionResponseSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_list_task_executions_serialize(
        self,
        workspace_uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if workspace_uuid is not None:
            
            _query_params.append(('workspace_uuid', workspace_uuid))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/cloud-node-workloads/executions',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_list_workloads(
        self,
        status: Optional[StrictStr] = None,
        profile_slug: Optional[StrictStr] = None,
        workspace_uuid: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[CloudNodeWorkloadSchema]:
        """List Workloads

        List all Cloud Node workloads visible to the authenticated user.  Optionally filter by status, profile_slug, or workspace_uuid.

        :param status:
        :type status: str
        :param profile_slug:
        :type profile_slug: str
        :param workspace_uuid:
        :type workspace_uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_workloads_serialize(
            status=status,
            profile_slug=profile_slug,
            workspace_uuid=workspace_uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[CloudNodeWorkloadSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_list_workloads_with_http_info(
        self,
        status: Optional[StrictStr] = None,
        profile_slug: Optional[StrictStr] = None,
        workspace_uuid: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[CloudNodeWorkloadSchema]]:
        """List Workloads

        List all Cloud Node workloads visible to the authenticated user.  Optionally filter by status, profile_slug, or workspace_uuid.

        :param status:
        :type status: str
        :param profile_slug:
        :type profile_slug: str
        :param workspace_uuid:
        :type workspace_uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_workloads_serialize(
            status=status,
            profile_slug=profile_slug,
            workspace_uuid=workspace_uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[CloudNodeWorkloadSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_list_workloads_without_preload_content(
        self,
        status: Optional[StrictStr] = None,
        profile_slug: Optional[StrictStr] = None,
        workspace_uuid: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Workloads

        List all Cloud Node workloads visible to the authenticated user.  Optionally filter by status, profile_slug, or workspace_uuid.

        :param status:
        :type status: str
        :param profile_slug:
        :type profile_slug: str
        :param workspace_uuid:
        :type workspace_uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_list_workloads_serialize(
            status=status,
            profile_slug=profile_slug,
            workspace_uuid=workspace_uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[CloudNodeWorkloadSchema]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_list_workloads_serialize(
        self,
        status,
        profile_slug,
        workspace_uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if status is not None:
            
            _query_params.append(('status', status))
            
        if profile_slug is not None:
            
            _query_params.append(('profile_slug', profile_slug))
            
        if workspace_uuid is not None:
            
            _query_params.append(('workspace_uuid', workspace_uuid))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/cloud-node-workloads',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_mark_workload_completed(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadSchema:
        """Mark Workload Completed

        Mark a workload as completed.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_mark_workload_completed_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_mark_workload_completed_with_http_info(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadSchema]:
        """Mark Workload Completed

        Mark a workload as completed.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_mark_workload_completed_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_mark_workload_completed_without_preload_content(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Mark Workload Completed

        Mark a workload as completed.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_mark_workload_completed_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_mark_workload_completed_serialize(
        self,
        uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads/{uuid}/complete',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_mark_workload_failed(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadSchema:
        """Mark Workload Failed

        Mark a workload as failed.  This can be used when a workload encounters an error during execution.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_mark_workload_failed_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_mark_workload_failed_with_http_info(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadSchema]:
        """Mark Workload Failed

        Mark a workload as failed.  This can be used when a workload encounters an error during execution.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_mark_workload_failed_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_mark_workload_failed_without_preload_content(
        self,
        uuid: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Mark Workload Failed

        Mark a workload as failed.  This can be used when a workload encounters an error during execution.

        :param uuid: (required)
        :type uuid: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_mark_workload_failed_serialize(
            uuid=uuid,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_mark_workload_failed_serialize(
        self,
        uuid,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads/{uuid}/fail',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_update_workload(
        self,
        uuid: StrictStr,
        cloud_node_workload_update_schema: CloudNodeWorkloadUpdateSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CloudNodeWorkloadSchema:
        """Update Workload

        Update a Cloud Node workload.  Can be used to update status or assign/unassign an instance.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_update_schema: (required)
        :type cloud_node_workload_update_schema: CloudNodeWorkloadUpdateSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_update_workload_serialize(
            uuid=uuid,
            cloud_node_workload_update_schema=cloud_node_workload_update_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_update_workload_with_http_info(
        self,
        uuid: StrictStr,
        cloud_node_workload_update_schema: CloudNodeWorkloadUpdateSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CloudNodeWorkloadSchema]:
        """Update Workload

        Update a Cloud Node workload.  Can be used to update status or assign/unassign an instance.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_update_schema: (required)
        :type cloud_node_workload_update_schema: CloudNodeWorkloadUpdateSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_update_workload_serialize(
            uuid=uuid,
            cloud_node_workload_update_schema=cloud_node_workload_update_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_update_workload_without_preload_content(
        self,
        uuid: StrictStr,
        cloud_node_workload_update_schema: CloudNodeWorkloadUpdateSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update Workload

        Update a Cloud Node workload.  Can be used to update status or assign/unassign an instance.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_update_schema: (required)
        :type cloud_node_workload_update_schema: CloudNodeWorkloadUpdateSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_update_workload_serialize(
            uuid=uuid,
            cloud_node_workload_update_schema=cloud_node_workload_update_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CloudNodeWorkloadSchema",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_update_workload_serialize(
        self,
        uuid,
        cloud_node_workload_update_schema,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if cloud_node_workload_update_schema is not None:
            _body_params = cloud_node_workload_update_schema


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v1/cloud-node-workloads/{uuid}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def src_app_api_cloud_node_workloads_workload_upload_results(
        self,
        uuid: StrictStr,
        cloud_node_workload_result_schema: CloudNodeWorkloadResultSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Workload Upload Results

        Handle the result of uploaded files for a Cloud Node workload.  This endpoint is called by the Cloud Node after files have been uploaded to signed URLs.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_result_schema: (required)
        :type cloud_node_workload_result_schema: CloudNodeWorkloadResultSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_workload_upload_results_serialize(
            uuid=uuid,
            cloud_node_workload_result_schema=cloud_node_workload_result_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def src_app_api_cloud_node_workloads_workload_upload_results_with_http_info(
        self,
        uuid: StrictStr,
        cloud_node_workload_result_schema: CloudNodeWorkloadResultSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Workload Upload Results

        Handle the result of uploaded files for a Cloud Node workload.  This endpoint is called by the Cloud Node after files have been uploaded to signed URLs.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_result_schema: (required)
        :type cloud_node_workload_result_schema: CloudNodeWorkloadResultSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_workload_upload_results_serialize(
            uuid=uuid,
            cloud_node_workload_result_schema=cloud_node_workload_result_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def src_app_api_cloud_node_workloads_workload_upload_results_without_preload_content(
        self,
        uuid: StrictStr,
        cloud_node_workload_result_schema: CloudNodeWorkloadResultSchema,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Workload Upload Results

        Handle the result of uploaded files for a Cloud Node workload.  This endpoint is called by the Cloud Node after files have been uploaded to signed URLs.

        :param uuid: (required)
        :type uuid: str
        :param cloud_node_workload_result_schema: (required)
        :type cloud_node_workload_result_schema: CloudNodeWorkloadResultSchema
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._src_app_api_cloud_node_workloads_workload_upload_results_serialize(
            uuid=uuid,
            cloud_node_workload_result_schema=cloud_node_workload_result_schema,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _src_app_api_cloud_node_workloads_workload_upload_results_serialize(
        self,
        uuid,
        cloud_node_workload_result_schema,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if uuid is not None:
            _path_params['uuid'] = uuid
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if cloud_node_workload_result_schema is not None:
            _body_params = cloud_node_workload_result_schema



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'CustomTokenAuthentication'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/cloud-node-workloads/{uuid}/upload-results',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# Copyright [2021-2025] Thanh Nguyen
# Copyright [2022-2023] [CNRS, Toward SAS]

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Common schema definition for robot description format conversion.

This module defines a unified intermediate representation that can capture
the semantics of different robot description formats while preserving
format-specific information through extensions.
"""

from __future__ import annotations
import math
import re
from copy import deepcopy
from dataclasses import dataclass, field
from enum import Enum
from typing import Any

import numpy as np


class JointType(Enum):
    """Supported joint types across formats."""

    REVOLUTE = "revolute"
    PRISMATIC = "prismatic"
    CONTINUOUS = "continuous"
    FIXED = "fixed"
    FLOATING = "floating"
    PLANAR = "planar"
    SPHERICAL = "spherical"  # SDF/USD specific
    UNIVERSAL = "universal"  # SDF/USD specific


class GeometryType(Enum):
    """Supported geometry types."""

    UNKNOWN = "unknown"
    BOX = "box"
    CYLINDER = "cylinder"
    SPHERE = "sphere"
    MESH = "mesh"
    PLANE = "plane"
    CAPSULE = "capsule"  # MJCF/SDF specific
    ELLIPSOID = "ellipsoid"  # SDF specific


class ActuatorType(Enum):
    """Supported actuator models."""

    DC_MOTOR = "dc_motor"  # Direct torque control
    SERVO = "servo"  # PD control
    VELOCITY = "velocity"  # Velocity damping control
    POSITION = "position"  # PD control
    TORQUE = "torque"  # Direct torque control
    MUSCLE = "muscle"  # Biomechanical muscle model


@dataclass
class Vector3:
    """3D vector representation."""

    x: float = 0.0
    y: float = 0.0
    z: float = 0.0

    def to_list(self) -> list[float]:
        """Convert to list format."""
        return [self.x, self.y, self.z]

    def to_array(self) -> np.ndarray:
        """Convert to numpy array."""
        return np.array([self.x, self.y, self.z])

    @classmethod
    def from_list(cls, values: list[float]) -> "Vector3":
        """Create from list of values."""
        return cls(values[0], values[1], values[2])


@dataclass
class Quaternion:
    """Quaternion representation for rotations."""

    x: float = 0.0
    y: float = 0.0
    z: float = 0.0
    w: float = 1.0

    def to_list(self) -> list[float]:
        """Convert to list format [x, y, z, w]."""
        return [self.x, self.y, self.z, self.w]

    @classmethod
    def from_rpy(cls, roll: float, pitch: float, yaw: float) -> "Quaternion":
        """Create quaternion from roll-pitch-yaw angles (Fixed XYZ convention)."""

        cy = math.cos(yaw * 0.5)
        sy = math.sin(yaw * 0.5)
        cp = math.cos(pitch * 0.5)
        sp = math.sin(pitch * 0.5)
        cr = math.cos(roll * 0.5)
        sr = math.sin(roll * 0.5)

        w = cr * cp * cy + sr * sp * sy
        x = sr * cp * cy - cr * sp * sy
        y = cr * sp * cy + sr * cp * sy
        z = cr * cp * sy - sr * sp * cy

        return cls(x, y, z, w)


@dataclass
class Pose:
    """6DOF pose representation."""

    position: Vector3 = field(default_factory=Vector3)
    orientation: Quaternion = field(default_factory=Quaternion)

    @classmethod
    def from_xyzrpy(cls, xyz: list[float], rpy: list[float]) -> "Pose":
        """Create pose from position and RPY orientation."""
        return cls(
            position=Vector3.from_list(xyz),
            orientation=Quaternion.from_rpy(rpy[0], rpy[1], rpy[2]),
        )


@dataclass
class Inertia:
    """Inertia tensor representation."""

    ixx: float = 0.0
    iyy: float = 0.0
    izz: float = 0.0
    ixy: float = 0.0
    ixz: float = 0.0
    iyz: float = 0.0

    def to_matrix(self) -> np.ndarray:
        """Convert to 3x3 inertia matrix."""
        return np.array(
            [
                [self.ixx, self.ixy, self.ixz],
                [self.ixy, self.iyy, self.iyz],
                [self.ixz, self.iyz, self.izz],
            ]
        )


@dataclass
class Material:
    """Material properties for visual and collision elements."""

    name: str | None = None
    color: list[float] | None = None  # RGBA
    texture: str | None = None
    specular: list[float] | None = None
    emissive: list[float] | None = None
    shininess: float | None = None


@dataclass
class Geometry:
    """Geometric shape definition."""

    type: GeometryType

    # Box parameters
    size: Vector3 | None = None

    # Cylinder parameters
    radius: float | None = None
    length: float | None = None

    # Sphere parameters
    # radius is shared

    # Mesh parameters
    filename: str | None = None
    scale: Vector3 | None = None

    # Format-specific extensions
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Visual:
    """Visual representation of a link."""

    name: str | None = None
    pose: Pose = field(default_factory=Pose)
    geometry: Geometry | None = None
    material: Material | None = None
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Collision:
    """Collision representation of a link."""

    name: str | None = None
    pose: Pose = field(default_factory=Pose)
    geometry: Geometry | None = None

    # Collision group reference (defined in CollisionConfig.groups)
    group: str = "default"

    # Contact properties
    mu_static: float | None = None
    mu_dynamic: float | None = None
    restitution: float | None = None
    stiffness: float | None = None
    damping: float | None = None

    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Link:
    """Robot link definition."""

    name: str

    # Inertial properties
    mass: float = 0.0
    center_of_mass: Vector3 = field(default_factory=Vector3)
    inertia: Inertia = field(default_factory=Inertia)

    # Geometric representations
    visuals: list[Visual] = field(default_factory=list)
    collisions: list[Collision] = field(default_factory=list)

    # Format-specific extensions
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class JointLimits:
    """Joint limit specification."""

    lower: float | None = None
    upper: float | None = None
    effort: float | None = None
    velocity: float | None = None

    # Additional limits for specific formats
    acceleration: float | None = None  # MJCF
    jerkmax: float | None = None  # MJCF


@dataclass
class JointDynamics:
    """Joint dynamics properties."""

    damping: float = 0.0
    friction: float = 0.0
    spring_reference: float = 0.0
    spring_stiffness: float = 0.0

    # Format-specific properties
    armature: float | None = None  # MJCF
    backlash: float | None = None


@dataclass
class Joint:
    """Robot joint definition."""

    name: str
    type: JointType
    parent_link: str
    child_link: str

    # Kinematic properties
    pose: Pose = field(default_factory=Pose)
    axis: Vector3 = field(default_factory=lambda: Vector3(0, 0, 1))

    # Constraints
    limits: JointLimits | None = None
    dynamics: JointDynamics | None = None

    # Safety and calibration
    safety_controller: dict[str, float] | None = None
    calibration: dict[str, float] | None = None

    # Default position
    home_position: float | None = None
    """Default position of the joint.
    
    In meters for prismatic joints, in radians for revolute joints.
    """

    # Format-specific extensions
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Actuator:
    """Actuator/motor definition."""

    name: str
    joint: str
    type: ActuatorType

    # Motor parameters
    torque_constant: float | None = None
    gear_ratio: float | None = None
    max_current: float | None = None
    max_torque: float | None = None
    max_velocity: float | None = None

    # Control parameters
    kp: float | None = None  # Proportional gain
    ki: float | None = None  # Integral gain
    kd: float | None = None  # Derivative gain

    # Physical properties
    resistance: float | None = None
    inductance: float | None = None
    efficiency: float | None = None

    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Sensor:
    """Sensor definition."""

    name: str
    type: str  # IMU, camera, lidar, force_torque, etc.
    parent_link: str
    pose: Pose = field(default_factory=Pose)

    # Update rate
    update_rate: float | None = None

    # Noise model
    noise: dict[str, float | list[float]] | None = None

    # Sensor-specific parameters
    parameters: dict[str, Any] = field(default_factory=dict)
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class ContactSurface:
    """Contact surface properties."""

    mu_static: float = 0.8
    mu_dynamic: float = 0.7
    restitution: float = 0.1
    stiffness: float = 10000.0
    damping: float = 20.0

    # Advanced contact parameters
    slip_compliance: dict[str, float] | None = None
    soft_cfm: float | None = None  # Constraint force mixing
    soft_erp: float | None = None  # Error reduction parameter


@dataclass
class Contact:
    """Contact definition for collision detection."""

    name: str
    link: str
    surface: ContactSurface = field(default_factory=ContactSurface)
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class PhysicsSolver:
    """Physics solver configuration."""

    type: str = "quick"  # quick, ode, bullet, dart, simbody
    iterations: int = 50
    sor: float = 1.3  # Successive Over-Relaxation parameter (ODE)
    min_step_size: float = 0.0001
    max_step_size: float = 0.001
    tolerance: float | None = None
    # Format-specific extensions
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Physics:
    """World-level physics properties."""

    gravity: Vector3 = field(default_factory=lambda: Vector3(0.0, 0.0, -9.81))
    timestep: float = 0.001
    solver: PhysicsSolver = field(default_factory=PhysicsSolver)
    contact: ContactSurface = field(default_factory=ContactSurface)
    # Format-specific extensions
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Scene:
    """Scene/render properties."""

    ambient: list[float] | None = None  # RGBA ambient light
    background: list[float] | None = None  # RGBA background color
    shadows: bool = True
    grid: bool = False
    origin_visual: bool = False
    # Format-specific extensions
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class CollisionGroup:
    """Named collision group with MuJoCo-style bitmask properties.

    Groups are defined in CollisionConfig and referenced by Collision.group.
    """

    name: str
    contype: int = 1  # Bitmask: what collision type(s) this group belongs to
    conaffinity: int = 1  # Bitmask: what collision type(s) this group collides with
    # To disable collisions, set both contype=0 and conaffinity=0


@dataclass
class CollisionExclude:
    """Exclude all collisions between two bodies (links).

    Maps to MuJoCo <contact><exclude body1="..." body2="..."/></contact>
    """

    body1: str  # Link name
    body2: str  # Link name


@dataclass
class CollisionPair:
    """Explicit geom-pair collision with optional contact overrides.

    Maps to MuJoCo <contact><pair geom1="..." geom2="..." .../></contact>

    Note: Bypasses parent-child filtering and contype/conaffinity checks.
    Geoms must be on different bodies (same-body collisions not allowed).
    """

    geom1: str  # Collision name (Collision.name) or link__collision format
    geom2: str  # Collision name (Collision.name) or link__collision format

    # Optional contact property overrides for this pair
    friction: list[float] | None = None  # [sliding, torsional, rolling]
    solref: list[float] | None = None  # [timeconst, dampratio]
    solimp: list[float] | None = None  # [dmin, dmax, width, mid, power]
    condim: int | None = None  # Contact dimensionality (1, 3, 4, 6)
    margin: float | None = None  # Distance margin for contact detection
    gap: float | None = None  # Initial gap between geoms

    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class CollisionConfig:
    """World-level collision configuration.

    Defines collision groups, body-level exclusions, and explicit geom pairs.
    """

    # Named collision groups (e.g., "default", "ROBOT", "ENV")
    groups: dict[str, CollisionGroup] = field(default_factory=dict)

    # Body-level exclusions (e.g., adjacent links, self-collision suppression)
    excludes: list[CollisionExclude] = field(default_factory=list)

    # Explicit geom pairs with optional contact overrides
    pairs: list[CollisionPair] = field(default_factory=list)

    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class Metadata:
    """Robot model metadata."""

    name: str
    version: str = "1.0"  # TODO(AI): What is this used for?
    author: str | None = None
    description: str | None = None
    source_format: str | None = None
    creation_date: str | None = None
    units: str = "SI"  # meters, kg, seconds, radians

    # Format-specific metadata
    extensions: dict[str, Any] = field(default_factory=dict)


@dataclass
class CommonSchema:
    """Unified robot description schema.

    This schema serves as an intermediate representation that can capture
    the semantics of different robot description formats while preserving
    format-specific information through extensions.

    The schema follows a hierarchical structure:
    - Metadata: Robot-level information
    - Physics: World-level physics properties (gravity, timestep, solver, contact)
    - Scene: Scene/render properties (ambient, background, shadows)
    - Links: Physical bodies with inertial, visual, and collision properties
    - Joints: Kinematic connections between links
    - Actuators: Motors and drive systems
    - Sensors: Sensing systems
    - Contacts: Contact surface definitions

    Extensions allow format-specific features to be preserved during
    conversion while maintaining compatibility across formats.
    """

    metadata: Metadata
    physics: Physics | None = None
    scene: Scene | None = None
    collision_config: CollisionConfig | None = None
    links: list[Link] = field(default_factory=list)
    joints: list[Joint] = field(default_factory=list)
    actuators: list[Actuator] = field(default_factory=list)
    sensors: list[Sensor] = field(default_factory=list)
    contacts: list[Contact] = field(default_factory=list)

    # Global extensions for format-specific features
    extensions: dict[str, Any] = field(default_factory=dict)

    def get_link(self, name: str) -> Link | None:
        """Get link by name."""
        for link in self.links:
            if link.name == name:
                return link
        return None

    def get_joint(self, name: str) -> Joint | None:
        """Get joint by name."""
        for joint in self.joints:
            if joint.name == name:
                return joint
        return None

    def get_actuator(self, name: str) -> Actuator | None:
        """Get actuator by name."""
        for actuator in self.actuators:
            if actuator.name == name:
                return actuator
        return None

    def get_root_links(self) -> list[Link]:
        """Get links that are not children of any joint (root links).

        Note: Joints with parent_link='world' are excluded from this calculation,
        so their child links are still considered roots.
        """
        child_links = {
            joint.child_link for joint in self.joints if joint.parent_link != "world"
        }
        return [link for link in self.links if link.name not in child_links]

    def get_single_root_link(self) -> Link:
        """Get the single canonical root link for this schema.

        This enforces the invariant that valid asset schemas have exactly one
        root link. Callers that rely on a well-defined base link (e.g. when
        composing robots into scenes) must use this helper instead of
        interpreting ``get_root_links()`` themselves.

        Raises:
            ValueError: If there are 0 or more than 1 root links.
        """
        roots = self.get_root_links()
        if len(roots) == 0:
            raise ValueError(
                f"Schema {self.metadata.name!r} has no root links "
                "(possible kinematic loop or malformed model)."
            )
        if len(roots) > 1:
            root_names = [link.name for link in roots]
            raise ValueError(
                f"Schema {self.metadata.name!r} has multiple root links: {root_names}. "
                "Expected exactly one canonical root link."
            )
        return roots[0]

    def get_kinematic_tree(self) -> dict[str, list[str]]:
        """Get kinematic tree structure as parent->children mapping."""
        tree = {}
        for joint in self.joints:
            parent = joint.parent_link
            child = joint.child_link
            if parent not in tree:
                tree[parent] = []
            tree[parent].append(child)
        return tree

    def validate(self) -> list[str]:
        """Validate schema consistency and return list of issues.

        Returns:
            List of validation error messages
        """
        issues = []

        # Check for duplicate names
        link_names = [link.name for link in self.links]
        if len(link_names) != len(set(link_names)):
            issues.append("Duplicate link names found.")

        joint_names = [joint.name for joint in self.joints]
        if len(joint_names) != len(set(joint_names)):
            issues.append("Duplicate joint names found.")

        # Check joint references
        # Note: 'world' is a special case - it's a valid parent reference but not an actual link
        for joint in self.joints:
            if joint.parent_link != "world" and not self.get_link(joint.parent_link):
                issues.append(
                    f"Joint {joint.name!r} references unknown parent link: {joint.parent_link}."
                )
            if not self.get_link(joint.child_link):
                issues.append(
                    f"Joint {joint.name!r} references unknown child link: {joint.child_link}."
                )

        # Check actuator references
        for actuator in self.actuators:
            if not self.get_joint(actuator.joint):
                issues.append(
                    f"Actuator {actuator.name!r} references unknown joint: {actuator.joint}."
                )

        # Check sensor references
        # Note: 'world' is a special case - it's a valid parent reference but not an actual link
        for sensor in self.sensors:
            if sensor.parent_link != "world" and not self.get_link(sensor.parent_link):
                issues.append(
                    f"Sensor {sensor.name!r} references unknown parent link: {sensor.parent_link}."
                )

        # Check for kinematic loops (basic check)
        try:
            roots = self.get_root_links()
            if len(roots) == 0:
                issues.append("No root links found - possible kinematic loop.")
        except Exception:
            issues.append("Error analyzing kinematic structure.")

        return issues

    def to_dict(self) -> dict:
        """Convert schema to dictionary representation.

        Uses cattrs for automatic recursive conversion. Vector3/Quaternion are
        serialized as canonical dicts ({"x": ..., "y": ..., "z": ...}).

        Returns:
            Dictionary representation suitable for JSON serialization
        """
        # Import here to avoid circular import (serialization imports schema types)
        from cyberwave_robot_format.serialization import converter

        return converter.unstructure(self)

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "CommonSchema":
        """Reconstruct CommonSchema from dictionary representation.

        Uses cattrs for automatic recursive conversion. Vector3/Quaternion must
        be dicts (list format is not accepted).

        Args:
            data: Dictionary as produced by to_dict()

        Returns:
            CommonSchema instance
        """
        # Import here to avoid circular import (serialization imports schema types)
        from cyberwave_robot_format.serialization import converter

        return converter.structure(data, cls)

    def merge_in(
        self,
        other: CommonSchema,
        instance_name: str,
        spawn_pose: Pose,
        fixed_base: bool = True,
    ) -> None:
        """Merge another schema into this one with deterministic namespacing.

        This method enables composition of multiple robot schemas into one by:
        1. Prefixing all names from `other` with `<instance_name>__` to avoid collisions
        2. Rewriting all internal references consistently
        3. Adding a world joint (FIXED or FREE) to position the merged robot in space

        Args:
            other: The schema to merge into this one
            instance_name: Prefix for all names from `other` (must match [A-Za-z0-9_]+)
            spawn_pose: Pose for spawning the merged robot in world frame (always required)
            fixed_base: If True, creates FIXED joint (robot bolted in place).
                       If False, creates FLOATING joint (6-DOF mobile robot like wheeled platform/drone)

        Raises:
            ValueError: If instance_name is invalid, or if ``other`` does not have
                exactly one root link.

        Example:
            >>> scene = CommonSchema(metadata=Metadata(name="scene"))
            >>> # Fixed-base robot arm
            >>> scene.merge_in(arm_schema, "arm1", spawn_pose=Pose(), fixed_base=True)
            >>> # Mobile wheeled robot
            >>> scene.merge_in(mobile_schema, "robot1",
            ...                spawn_pose=Pose(position=Vector3(2, 0, 0)),
            ...                fixed_base=False)

        Note:
            This method mutates `self`. The `other` schema is deep-copied before modification.
            Cross-robot attachments (e.g., arm-on-platform) are not yet supported but can be
            added in the future via an additional parameter.
        """
        # Validate instance_name
        if not instance_name:
            raise ValueError("instance_name cannot be empty.")
        if not re.match(r"^[A-Za-z0-9_]+$", instance_name):
            raise ValueError(
                f"instance_name must match [A-Za-z0-9_]+, got: {instance_name!r}"
            )

        # Determine root link for spawn joint (must be unique)
        root_link = other.get_single_root_link()

        # Deep copy to avoid mutating the original
        other_copy = deepcopy(other)

        # Build name mapping
        prefix = f"{instance_name}__"

        def prefixed(name: str) -> str:
            """Apply prefix to a name."""
            return f"{prefix}{name}"

        # Rewrite all links
        for link in other_copy.links:
            link.name = prefixed(link.name)

        # Rewrite all joints and their references
        for joint in other_copy.joints:
            joint.name = prefixed(joint.name)
            if joint.parent_link != "world":
                joint.parent_link = prefixed(joint.parent_link)
            joint.child_link = prefixed(joint.child_link)

        # Rewrite all actuators and their references
        for actuator in other_copy.actuators:
            actuator.name = prefixed(actuator.name)
            actuator.joint = prefixed(actuator.joint)

        # Rewrite all sensors and their references
        for sensor in other_copy.sensors:
            sensor.name = prefixed(sensor.name)
            if sensor.parent_link != "world":
                sensor.parent_link = prefixed(sensor.parent_link)

        # Rewrite all contacts and their references
        for contact in other_copy.contacts:
            contact.name = prefixed(contact.name)
            contact.link = prefixed(contact.link)

        # Merge collision_config with prefixing
        if other_copy.collision_config:
            # Ensure self has collision_config
            if self.collision_config is None:
                self.collision_config = CollisionConfig()

            # Merge groups (copy without prefixing group names - they're global identifiers)
            for group_name, group in other_copy.collision_config.groups.items():
                if group_name not in self.collision_config.groups:
                    self.collision_config.groups[group_name] = group

            # Merge excludes with prefixed body names
            for exclude in other_copy.collision_config.excludes:
                prefixed_exclude = CollisionExclude(
                    body1=prefixed(exclude.body1), body2=prefixed(exclude.body2)
                )
                self.collision_config.excludes.append(prefixed_exclude)

            # Merge pairs with prefixed geom names
            for pair in other_copy.collision_config.pairs:
                prefixed_pair = CollisionPair(
                    geom1=prefixed(pair.geom1),
                    geom2=prefixed(pair.geom2),
                    friction=pair.friction,
                    solref=pair.solref,
                    solimp=pair.solimp,
                    condim=pair.condim,
                    margin=pair.margin,
                    gap=pair.gap,
                    extensions=pair.extensions,
                )
                self.collision_config.pairs.append(prefixed_pair)

        # Append all entities to self
        self.links.extend(other_copy.links)
        self.joints.extend(other_copy.joints)
        self.actuators.extend(other_copy.actuators)
        self.sensors.extend(other_copy.sensors)
        self.contacts.extend(other_copy.contacts)

        # Add spawn joint (always required)
        spawn_joint = Joint(
            name=prefixed("spawn"),
            type=JointType.FIXED if fixed_base else JointType.FLOATING,
            parent_link="world",
            child_link=prefixed(root_link.name),
            pose=spawn_pose,
        )
        self.joints.append(spawn_joint)



"""
Keyboard teleoperation for controlling twins from the terminal.
"""

from __future__ import annotations

import os
import sys
import time
from dataclasses import dataclass
from typing import TYPE_CHECKING, Any, Dict, List

if TYPE_CHECKING:
    from .twin import Twin


class KeyboardBindings:
    """Builder for keyboard teleoperation bindings."""

    def __init__(self) -> None:
        self._bindings: List[Dict[str, Any]] = []

    def bind(
        self, key: str, joint_name: str, direction: str = "increase"
    ) -> "KeyboardBindings":
        """
        Bind a key to a joint action.

        Args:
            key: Keyboard key (e.g., "W", "S")
            joint_name: Name of the joint to control
            direction: "increase" or "decrease"

        Returns:
            Self for chaining
        """
        if direction not in {"increase", "decrease"}:
            raise ValueError("direction must be 'increase' or 'decrease'")
        if not key:
            raise ValueError("key is required")
        if not joint_name:
            raise ValueError("joint_name is required")
        self._bindings.append(
            {
                "key": key.upper(),
                "jointName": joint_name,
                "direction": direction,
            }
        )
        return self

    def build(self) -> List[Dict[str, Any]]:
        """Return the bindings as a list of dictionaries."""
        return list(self._bindings)


@dataclass
class _Binding:
    joint_name: str
    direction: str


class KeyboardTeleop:
    """
    CLI keyboard teleoperation helper.

    Reads keyboard input and updates joint positions via the twin's joints controller.
    """

    def __init__(
        self,
        twin: "Twin",
        bindings: List[Dict[str, Any]],
        *,
        step: float = 0.05,
        rate_hz: int = 20,
        fetch_initial: bool = True,
        verbose: bool = True,
    ) -> None:
        """
        Initialize keyboard teleop.

        Args:
            twin: Twin instance to control
            bindings: List of key bindings from KeyboardBindings.build()
            step: Position change per keypress (degrees)
            rate_hz: Polling rate in Hz
            fetch_initial: Whether to fetch initial joint positions
            verbose: Whether to print status messages
        """
        self._twin = twin
        self._bindings = self._normalize_bindings(bindings)
        self._step = float(step)
        self._interval = 1.0 / float(rate_hz) if rate_hz > 0 else 0.05
        self._fetch_initial = fetch_initial
        self._verbose = verbose
        self._positions: Dict[str, float] = {}

    def run(self, stop_key: str = "q") -> None:
        """
        Run the keyboard teleop loop.

        Args:
            stop_key: Key to press to stop (default: 'q')
        """
        if not self._bindings:
            raise ValueError("No keyboard bindings configured")
        if self._fetch_initial:
            self._positions = self._get_initial_positions()

        stop_key = stop_key.lower()
        if self._verbose:
            print(f"Keyboard teleop active. Press '{stop_key}' to stop.")
            print("Bindings:")
            for key, bindings in self._bindings.items():
                for b in bindings:
                    print(f"  {key}: {b.joint_name} ({b.direction})")

        if os.name == "nt":
            self._run_windows(stop_key)
        else:
            self._run_posix(stop_key)

    def _normalize_bindings(self, bindings: List[Dict[str, Any]]) -> Dict[str, List[_Binding]]:
        normalized: Dict[str, List[_Binding]] = {}
        for binding in bindings:
            if not isinstance(binding, dict):
                continue
            key = str(binding.get("key") or "").upper()
            joint_name = binding.get("jointName") or binding.get("joint_name")
            direction = binding.get("direction") or "increase"
            if not key or not joint_name:
                continue
            normalized.setdefault(key, []).append(
                _Binding(joint_name=str(joint_name), direction=str(direction))
            )
        return normalized

    def _get_initial_positions(self) -> Dict[str, float]:
        positions: Dict[str, float] = {}
        try:
            positions = self._twin.joints.get_all()
        except Exception:
            pass
        return positions

    def _apply_binding(self, binding: _Binding) -> None:
        delta = self._step if binding.direction == "increase" else -self._step
        current = self._positions.get(binding.joint_name, 0.0)
        next_pos = current + delta
        self._positions[binding.joint_name] = next_pos
        try:
            self._twin.joints.set(binding.joint_name, next_pos, degrees=True)
            if self._verbose:
                print(f"  {binding.joint_name}: {next_pos:.2f}°")
        except Exception as e:
            if self._verbose:
                print(f"  Error setting {binding.joint_name}: {e}")

    def _run_windows(self, stop_key: str) -> None:
        import msvcrt

        while True:
            if msvcrt.kbhit():
                key = msvcrt.getch().decode("utf-8", errors="ignore").lower()
                if not key:
                    continue
                if key == stop_key:
                    break
                bindings = self._bindings.get(key.upper(), [])
                for binding in bindings:
                    self._apply_binding(binding)
            time.sleep(self._interval)

    def _run_posix(self, stop_key: str) -> None:
        import select
        import termios
        import tty

        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setcbreak(fd)
            while True:
                rlist, _, _ = select.select([sys.stdin], [], [], self._interval)
                if rlist:
                    key = sys.stdin.read(1).lower()
                    if key == stop_key:
                        break
                    bindings = self._bindings.get(key.upper(), [])
                    for binding in bindings:
                        self._apply_binding(binding)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)



